--- LibCombatDetails-1.0 was developed to help speed up analysis of COMBAT_LOG_EVENT argument parsing. 
-- Due to the nature of the event, the arguments sent with this event often change, for instance, the variable
-- that tells you if a melee swing crit is in argument # 15, but for a spell crit it is argument #18, and for a heal
-- crit, it is #14. This library is developed to parse the arguments and make it straightforward and easy to obtain
-- the information you need.
-- 
-- Due to the design of the wow-event and this library, most functions are used the same way, below is a usage example
-- that will give you an idea of how to use any function in this library which uses the (...) prototype.
-- 
-- USAGE INSTRUCTIONS:
-- 
-- <pre>local Combat = LibStub("LibCombatDetails-1.0");
-- local MyModFrame = CreateFrame("frame", "MyModFrame", UIParent);
-- MyModFrame:SetScript("OnEvent", function(event, ...) HandleEvent(event, ...) end);
-- MyModFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
-- 
-- function HandleEvent(event, ...)
--      if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
--           HandleCombatEvent(event, ...);
--      end
-- end
--
-- function HandleCombatEvent(e, ...)
--      if (Combat:IsMelee(...)) then
--           if (Combat:IsDamage(...)) then
--                DEFAULT_CHAT_FRAME:AddMessage("Melee Swing: "..Combat:GetDamage(...));
--           else
--                DEFAULT_CHAT_FRAME:AddMessage("Melee Miss!");
--           end
--      end
-- end</pre>
-- @class file
-- @name LibCombatDetails-1.0
-- @release v0.11-beta

--[[
	Copyright (c) 2009 Aren (ReAn/Voorije) Blondahl

	Permission is hereby granted, free of charge, to any person obtaining a copy
	of this software and associated documentation files (the "Software"), to deal
	in the Software without restriction, including without limitation the rights
	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
	copies of the Software, and to permit persons to whom the Software is
	furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
	THE SOFTWARE.
--]]
local MAJOR, MINOR = "LibCombatDetails-1.0", (tonumber("12") or 9999999)

assert(LibStub, "LibCombatDetails-1.0 Requires LibStub");

local Combat = LibStub:NewLibrary(MAJOR, MINOR)

if not Combat then return end

local function BeginsWith(str, tok)
	local _s = string.find(str, tok);
	if (_s == 1) then
		return true;
	else
		return false;
	end
end

local function EndsWith(str, tok)
	local rstr = string.reverse(str);
	local rtok = string.reverse(tok);
	
	local _s = string.find(rstr, rtok);
	if (_s == 1) then
		return true;
	else
		return false;
	end
end

--- Gets the combat-event that occured
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return event - The Event Code
function Combat:GetEvent(...)
	local _, event = ...;
	
	return event;
end

--- Gets the source of this event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return srcGUID - The GUID of the source unit of this event.
-- @return srcName - The name of the source unit of this event.
-- @return srcFlags - The flags of the source unit of this event.
function Combat:GetSource(...)
	local _, _, srcGUID, srcName, srcFlags = ...;
	
	return srcGUID, srcName, srcFlags;
end

--- Gets the destinations of this event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return dstGUID - The GUID of the destination unit of this event.
-- @return dstName - The name of the destination unit of this event.
-- @return dstFlags - The flags of the destination unit of this event.
function Combat:GetDest(...)
	local _, _, _, _, _, dstGUID, dstName, dstFlags = ...;
	
	return dstGUID, dstName, dstFlags;
end

--- Determines if the source of this event is the player
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return fromPlayer - A boolean value representing if the source was the player.
function Combat:FromPlayer(...)
	local srcGUID = self:GetSource(...);
	
	return (srcGUID == UnitGUID("player"));
end

--- Determines if the destination of this event is the player
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return toPlayer - A boolean value representing if the destination was the player.
function Combat:ToPlayer(...)
	local dstGUID = self:GetDest(...);
	
	return (dstGUID == UnitGUID("player"));
end

--- Determines if the source of this event is the target
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return fromTarget - A boolean value representing if the source was the current target.
function Combat:FromTarget(...)
	local srcGUID = self:GetSource(...);
	
	return (srcGUID == UnitGUID("target"));
end

--- Determines if the destination of this event is the target
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return toTarget - A boolean value representing if the destination was the current target.
function Combat:ToTarget(...)
	local dstGUID = self:GetDest(...);
	
	return (dstGUID == UnitGUID("target"));
end

--- Determines if the source of this event is the focus
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return fromFocus - A boolean value representing if the source was the focus target.
function Combat:FromFocus(...)
	local srcGUID = self:GetSource(...);
	
	return (srcGUID == UnitGUID("focus"));
end

--- Determines if the destination of this event is the focus
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return toFocus - A boolean value representing if the destination was the focus target.
function Combat:ToFocus(...)
	local dstGUID = self:GetDest(...);
	
	return (dstGUID == UnitGUID("focus"));
end

--- Determines if this event is a MELEE event.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return isMelee - A boolean value representing if the event was a melee swing. (White Hit)
function Combat:IsMelee(...)
	local _, event = ...;
	
	local _s, _e = string.find(event, "SWING");
	
	if (_s == 1 and _e == 5) then
		return true;
	else
		return false;
	end
end

--- Determines if this event is a RANGED event.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return isRanged - A boolean value representing if the event was a ranged hit. (White hit)
function Combat:IsRanged(...)
	local _, event = ...;
	
	local _s, _e = string.find(event, "RANGED");
	
	if (_s == 1 and _e == 6) then
		return true;
	else
		return false;
	end
end

--- Determines if this event is a SPELL event.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return isSpell - A boolean value representing if the event was a spell/skill. (Yellow Hit)
function Combat:IsSpell(...)
	local _, event = ...;
	
	local _s, _e = string.find(event, "SPELL");
	
	if (_s == 1 and _e == 5) then
		return true;
	else
		return false;
	end
end

--- Determines if this event is a PERIODIC SPELL event.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return isPeriodicSpell - A boolean value representing if the event was a periodic spell/skill. (DoT)
function Combat:IsPeriodic(...)
	local _, event = ...;
	
	local _s, _e = string.find(event, "SPELL_PERIODIC");
	
	if (_s == 1 and _e == 14) then
		return true;
	else
		return false;
	end
end

--- Determines if this event is a BUILDING SPELL event.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return isBuildingSpell - A boolean value representing if the event was a spell/skill that can damage buildings.
function Combat:IsBuilding(...)
	local _, event = ...;
	
	local _s, _e = string.find(event, "SPELL_BUILDING");
	
	if (_s == 1 and _e == 14) then
		return true;
	else
		return false;
	end
end

--- Determines if this event is a ENVIRONMENTAL event.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return isSpell - A boolean value representing if the event was environmental.
function Combat:IsEnvironmental(...)
	local _, event = ...;
	
	local _s, _e = string.find(event, "ENVIRONMENTAL");
	
	if (_s == 1) then
		return true;
	else
		return false;
	end
end

--- Gets the suffix of the combat event
-- When this function is executed, it determines the prefix type and gets everything after it.
-- @returns The suffix of the event without the leading _
function Combat:GetSuffix(...)
	local _, event = ...;
	
	if (self:IsMelee(...)) then
		return string.sub(event, 7);
	elseif (self:IsRanged(...)) then
		return string.sub(event, 8);
	elseif (self:IsEnvironmental(...)) then
		return string.sub(event, 15); 
	elseif (self:IsBuilding(...)) then
		return string.sub(event, 16);
	elseif (self:IsPeriodic(...)) then
		return string.sub(event, 16);
	elseif (self:IsSpell(...)) then
		return string.sub(event, 7);
	else
		return nil;
	end
end

--- Checks if the suffix matches the input
-- @param suf The suffix to check against
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @returns True/False depending on the match.
function Combat:IsSuffix(suf, ...)
	if (self:GetSuffix(...) == suf) then
		return true;
	else
		return false;
	end
end

--- Returns the spell information of the event.
-- This function will only return spell info for the 
-- 	RANGED_, 
-- 	SPELL_, 
-- 	SPELL_PERIODIC_, 
-- 	SPELL_BUILDING_,
-- prefix types.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return SpellID - The Spell ID# of the spell
-- @return SpellName - The Spell Name of the spell
-- @return SpellSchool - The Spell School of the spell
function Combat:GetSpellDetails(...)
	if (self:IsMelee(...) or
		self:IsEnvironmental(...)) then
		return nil;
	end
	
	local _, _, _, _, _, _, _, _, spellID, spellName, spellSchool = ...;
	
	return spellID, spellName, spellSchool;
end

--[[ Functions for _DAMAGE Suffix ]]--

--- Determins if this event is a _DAMAGE suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsDamage(...)
	return self:IsSuffix("DAMAGE", ...);
end

--- Returns the additional parameters for _DAMAGE suffixed events.
-- This function adjusts accordingly based on the prefix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return damage - The Damage done on this event
-- @return overkill - The Damage done that exceeded the health of the unit
-- @return school - The school of damage that was done
-- @return resisted - The amount resisted by the target
-- @return blocked - The amount blocked by the target
-- @return absorbed - The amount absorbed by the target (nil if 0)
-- @return critical - If the event was a critical damage event (nil or 1)
-- @return glancing - If the event was a glancing damage event (nil or 1)
-- @return crushing - If the event was a crushing damage event (nil or 1)
function Combat:GetDamageInfo(...)
	if (not self:IsDamage(...)) then return nil end

	if (self:IsMelee(...)) then
		local _,_,_,_,_,_,_,_, damage, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = ...;
		
		return damage, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing;
	else
		local _,_,_,_,_,_,_,_,_,_,_, damage, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = ...;
		
		return damage, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing;
	end
end

--- Returns the Damage Component of a Damage Event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetDamage(...)
	local damage = self:GetDamageInfo(...);
	
	return damage;
end

--- Returns the Overkill Component of a Damage Event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetOverkill(...)
	local d, overkill = self:GetDamageInfo(...);
	
	if (d) then
		return overkill;
	else
		return nil;
	end
end

--- Returns the School Component of a Damage Event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetSchool(...)
	local d, _, school = self:GetDamageInfo(...);
	
	if (d) then
		return school;
	else
		return nil;
	end
end

--- Returns the Resisted Component of a Damage Event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetResisted(...)
	local d, _, _, resisted = self:GetDamageInfo(...);
	
	if (d) then
		return resisted;
	else
		return nil;
	end
end

--- Returns the Blocked Component of a Damage Event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetBlocked(...)
	local d, _, _, _, blocked = self:GetDamageInfo(...);
	
	if (d) then
		return blocked;
	else
		return nil;
	end
end

---Returns the Absorbed Component of a Damage Event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetAbsorbed(...)
	local d, _, _, _, _, absorb = self:GetDamageInfo(...);
	
	if (d) then
		if (abosrb == nil) then absorb = 0 end
		return absorb;
	else
		return nil;
	end
end

--- Determines if this damage/heal event was a critical
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsCritical(...)
	local _suffix = self:GetSuffix(...);
	
	if (_suffix == "DAMAGE") then
		local d, _, _, _, _, _, crit = self:GetDamageInfo(...);
		
		if (d) then
			if (crit == 1) then
				return true;
			else
				return false;
			end
		else
			return nil;
		end
	elseif (_suffix == "HEAL") then
		local h, _, crit = self:GetHealInfo(...);
		
		if (h) then
			if (crit == 1) then
				return true;
			else
				return false;
			end
		else
			return nil;
		end
	end
end

--- Determines if this damage event is a glancing blow
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsGlancing(...)
	local d, _, _, _, _, _, _, glance = self:GetDamageInfo(...);
	
	if (d) then
		if (glance == 1) then
			return true;
		else
			return false;
		end
	else
		return nil;
	end
end

--- Determins if this damage event is a crushing blow
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsCrushing(...)
	local d, _, _, _, _, _, _, _, crush = self:GetDamageInfo(...);
	
	if (d) then
		if (crush == 1) then
			return true;
		else
			return false;
		end
	else
		return nil;
	end
end

--[[ Functions for _HEAL Suffix ]]--

-- Determines if this event is a HEAL suffix event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsHeal(...)
	if (self:GetSuffix(...) == "HEAL") then
		return true;
	else
		return false;
	end
end

--- Returns the additional parameters for _HEAL suffixed events.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return heal - The Healing done by this event
-- @return overheal - The Healing done that exceeded the health of the unit
-- @return critical - If the event was a critical damage event (nil or 1)
function Combat:GetHealInfo(...)
	if (not self:IsHeal(...)) then return nil end
	
	local _,_,_,_,_,_,_,_,_,_,_, heal, overheal, crit = ...;
	
	return heal, overheal, crit;
end

--- Returns the Healing Component of this heal event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetHeal(...)
	if (not self:IsHeal(...)) then return nil end
	
	local heal = self:GetHealInfo(...);
	return heal;
end

--- Returns the Overheal Component of this heal event
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:GetOverheal(...)
	if (not self:IsHeal(...)) then return nil end
	
	local _, overheal = self:GetHealInfo(...);
	return overheal;
end

--[[ Functions for _MISSED Suffix ]]--

-- Determines if this event was a _MISSED suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsMiss(...)
	return self:IsSuffix("MISSED", ...);
end

--- Returns the additional parameters for _MISSED suffixed events.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return missType - The Miss Type (DODGED, MISS, PARRY)
-- @return missamount - The amount missed (for partial misses)
function Combat:GetMissInfo(...)
	if (not self:IsMiss(...)) then return nil end
	
	if (self:IsMelee(...)) then
		local _,_,_,_,_,_,_,_, missType, missamount = ...;
		
		return missType, missamount;
	else
		local _,_,_,_,_,_,_,_,_,_,_, missType, missamount = ...;
		
		return missType, missamount;
	end
end

--[[ Functions for _ENERGIZE Suffix ]]--

--- Determines if this event is an _ENERGIZE suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsEnergize(...)
	return self:IsSuffix("ENERGIZE", ...);
end

--- Returns the additional parameters for _ENERGIZE suffixed events.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return amount - The amount energized
-- @return power - The type of energize
function Combat:GetEnergizeInfo(...)
	if (not self:IsEnergize(...)) then return nil end
	
	local _,_,_,_,_,_,_,_,_,_,_,amount, power = ...;
	
	return amount, power;
end

--- Determines if this event is an _DRAIN suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsDrain(...)
	return self:IsSuffix("DRAIN", ...);
end

--- Determines if this event is an _LEECH suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsLeech(...)
	return self:IsSuffix("LEECH", ...);
end

--- Returns the additional parameters for _DRAIN and _LEECH suffixed events.
-- Both _DRAIN and _LEECH have the same argument prototype, this does the dirty work.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return amount - The amount drained/leeched
-- @return power - The type of drain/leech
-- @return extra - The extra spell linked with this event
function Combat:GetDrainLeechInfo(...)
	local _suffix = self:GetSuffix(...);
	if (_suffix ~= "DRAIN" and _suffix ~= "LEECH") then return nil end -- saves a GetSuffix call
	
	local _,_,_,_,_,_,_,_,_,_,_,amount, power, extra = ...;
	
	return amount, power, extra;
end

--- Returns the additional parameters for _DRAIN suffixed events.
-- Shortcut method.
-- @see Combat:GetDrainLeechInfo(...)
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return amount - The amount drained/leeched
-- @return power - The type of drain/leech
-- @return extra - The extra spell linked with this event
function Combat:GetDrainInfo(...)
	return self:GetDrainLeechInfo(...);
end

--- Returns the additional parameters for _LEECH suffixed events.
-- Shortcut method.
-- @see Combat:GetDrainLeechInfo(...)
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return amount - The amount drained/leeched
-- @return power - The type of drain/leech
-- @return extra - The extra spell linked with this event
function Combat:GetLeechInfo(...)
	return self:GetDrainLeechInfo(...);
end

--[[ Functions for _INTERUPT and _DISPEL_FAILED suffix ]]--

--- Determines if this event is an _INTERUPT suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsInterupt(...)
	return self:IsSuffix("INTERUPT", ...);
end

--- Determines if this event is an _DISPEL_FAILED suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsDispelFail(...)
	return self:IsSuffix("DISPEL_FAILED", ...);
end

--- Returns the additional parameters for _INTERUPT and _DISPEL_FAILED suffixed events.
-- Both _INTERUPT and _DRAIN_FAILED have the same argument prototype, this does the dirty work.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return extraID - The Extra Spell ID
-- @return extraName - The Extra Spell Name
-- @return extraSchool - The Extra Spell School
function Combat:GetInterFailInfo(...)
	if (not self:IsInterupt(...) and not self:IsDispelFail(...)) then return nil end
	
	local _,_,_,_,_,_,_,_,_,_,_, extraID, extraName, extraSchool;
	
	return extraID, extraName, extraSchool;
end

--- Returns the additional parameters for _INTERUPT suffixed events.
-- Shortcut method.
-- @see Combat:GetInterFailInfo(...)
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return extraID - The Extra Spell ID
-- @return extraName - The Extra Spell Name
-- @return extraSchool - The Extra Spell School
function Combat:GetInteruptInfo(...)
	return self:InterFailInfo(...);
end

--- Returns the additional parameters for _DISPEL_FAILED suffixed events.
-- Shortcut method.
-- @see Combat:GetInterFailInfo(...)
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return extraID - The Extra Spell ID
-- @return extraName - The Extra Spell Name
-- @return extraSchool - The Extra Spell School
function Combat:GetDispelFailInfo(...)
	return self:InterFailInfo(...);
end

--[[ Functions for _DISPEL / _STOLEN suffix ]]--

--- Determines if this event is a _DISPEL suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsDispel(...)
	return self:IsSuffix("DISPEL", ...);
end

--- Determines if this event is a _STOLEN suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsStolen(...)
	return self:IsSuffix("STOLEN", ...);
end

--- Returns the additional parameters for _STOLEN and _DISPEL suffixed events.
-- Both _STOLEN and _DISPEL have the same argument prototype, this does the dirty work.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return extraID - The Extra Spell ID
-- @return extraName - The Extra Spell Name
-- @return extraSchool - The Extra Spell School
-- @return aura - The Aura linked to the event
function Combat:GetDispelStolenInfo(...)
	if (not self:IsDispel(...) and self:IsStolen(...)) then return nil end
	
	local _,_,_,_,_,_,_,_,_,_,_, extraID, extraName, extraSchool, aura = ...;
	
	return extraID, extraName, extraSchool, aura;
end

--- Returns the additional parameters for _STOLEN suffixed events.
-- Shortcut method.
-- @see Combat:GetDispelStolentInfo(...)
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return extraID - The Extra Spell ID
-- @return extraName - The Extra Spell Name
-- @return extraSchool - The Extra Spell School
-- @return aura - The Aura linked to the event
function Combat:GetDispelInfo(...)
	return self:GetDispelStolenInfo(...);
end

--- Returns the additional parameters for _DISPEL suffixed events.
-- Shortcut method.
-- @see Combat:GetDispelStolentInfo(...)
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return extraID - The Extra Spell ID
-- @return extraName - The Extra Spell Name
-- @return extraSchool - The Extra Spell School
-- @return aura - The Aura linked to the event
function Combat:GetStolenInfo(...)
	return self:GetDispelStolenInfo(...);
end

--[[ Functions for _AURA_ Suffixes ]]--

--- Determines if this event is an AURA event matching the following event suffixes:
-- AURA_APPLIED
-- AURA_REMOVED
-- AURA_APPLIED_DOSE
-- AURA_REMOVED_DOSE
-- AURA_REFRESH
-- AURA_BROKEN
-- AURA_BROKEN_SPELL
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAura(...)
	local _suffix = self:GetSuffix(...);
	
	if (_suffix == "AURA_APPLIED") then return true end
	if (_suffix == "AURA_REMOVED") then return true end
	if (_suffix == "AURA_APPLIED_DOSE") then return true end
	if (_suffix == "AURA_REMVOED_DOSE") then return true end
	if (_suffix == "AURA_REFRESH") then return true end
	if (_suffix == "AURA_BROKEN") then return true end
	if (_suffix == "AURA_BROKEN_SPELL") then return true end
	
	return false;
end

--- Determines if this event is a _AURA_APPLIED suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAuraApplied(...)
	return self:IsSuffix("AURA_APPLIED", ...);
end

--- Determines if this event is a _AURA_REMOVED suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAuraRemoved(...)
	return self:IsSuffix("AURA_REMOVED", ...);
end

--- Determines if this event is a _AURA_APPLIED_DOSE suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAuraAppliedDose(...)
	return self:IsSuffix("AURA_APPLIED_DOSE", ...);
end

--- Determines if this event is a _AURA_REMOVED_DOSE suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAuraRemovedDose(...)
	return self:IsSuffix("AURA_REMOVED_DOSE", ...);
end

--- Determines if this event is a _AURA_REFRESH suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAuraRefresh(...)
	return self:IsSuffix("AURA_REFRESH", ...);
end

--- Determines if this event is a _AURA_BROKEN suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAuraBroken(...)
	return self:IsSuffix("AURA_BROKEN", ...);
end

--- Determines if this event is a _AURA_BROKEN_SPELL suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsAuraBrokenSpell(...)
	return self:IsSuffix("AURA_BROKEN_SPELL", ...);
end

--- Returns the additional parameters for all AURA suffixed events.
-- This function has three return prototypes:
-- For _APPLIED_DOSE & _REMOVED_DOSE:
-- 	auraType, amount
-- For _BROKEN_SPELL:
-- 	extraID, extraName, extraSchoo, aura
-- For the rest:
-- 	auraType
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return extraID - The Extra Spell ID
-- @return extraName - The Extra Spell Name
-- @return extraSchool - The Extra Spell School
-- @return aura - The Aura linked to the event
-- @return auraType - The Aura-Type
-- @return amount - the Dose Applied/Removed to the aura
function Combat:GetAuraInfo(...)
	if (not self:IsAura(...)) then return nil end
	
	if (self:IsAuraAppliedDose(...) or self:IsRemovedDose(...)) then
		local _,_,_,_,_,_,_,_,_,_,_, auraType, amount = ...;
		return auraType, amount;
	elseif (self:IsAuraBrokenSpell(...)) then
		local _,_,_,_,_,_,_,_,_,_,_, exID, exName, exSchool, aura = ...;
		return exID, exName, exSchool, aura;
	else
		local _,_,_,_,_,_,_,_,_,_,_, auraType = ...;
		return auraType;
	end
end

--[[ Functions for Casting Events ]]--

--- Determines if this event is a _CAST_START suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsCastStart(...)
	return self:IsSuffix("CAST_START", ...);
end

--- Determines if this event is a _CAST_SUCCESS suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsCastSuccess(...)
	return self:IsSuffix("CAST_SUCCESS", ...);
end

--- Determines if this event is a _CAST_FAILED suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsCastFailed(...)
	return self:IsSuffix("CAST_FAILED", ...);
end

--- Gets the Fail Type for _CAST_FAILED Events
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return failType - The Fail Type for this cast
function Combat:GetCastFailType(...)
	if (not self:IsCastFailed(...)) then return nil end
	
	local _,_,_,_,_,_,_,_,_,_,_, failType = ...;
	
	return failType;
end

--[[ Functions for Extra Attacks ]]--

--- Determines if this event is a _EXTRA_ATTACKS suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsExtraAttacks(...)
	return self:IsSuffix("EXTRA_ATTACKS");
end

--- Gets the Number of extra attacks for a _EXTRA_ATTACKS event.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
-- @return number - The number of extra attacks.
function Combat:GetExtraAttacks(...)
	if (not self:IsExtraAttacks(...)) then return nil end
	
	return select(9, ...);
end

--[[ Functions for the rest! ]]--

--- Determines if this event is a _INSTAKILL suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsInstakill(...)
	return self:IsSuffix("INSTAKILL", ...);
end

--- Determines if this event is a _DURABILITY_DAMAGE suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsDurability(...)
	return self:IsSuffix("DURABILITY_DAMAGE", ...);
end

--- Determines if this event is a _DURABILITY_DAMAGE_ALL suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsDurabilityAll(...)
	return self:IsSuffix("DURABILITY_DAMAGE_ALL", ...);
end

--- Determines if this event is a _CREATE suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsCreate(...)
	return self:IsSuffix("CREATE", ...);
end

--- Determines if this event is a _SUMMON suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsSummon(...)
	return self:IsSuffix("SUMMON", ...);
end

--- Determines if this event is a _RESSURECT suffix.
-- @param ... Pass the '...' from your COMBAT_LOG_EVENT event.
function Combat:IsRessurect(...)
	return self:IsSuffix("RESSURECT", ...);
end
